const fn iterations(bits: u32) -> u32Expand description
Calculate the maximum number of iterations required according to safegcd-bounds: https://github.com/sipa/safegcd-bounds
const fn iterations(bits: u32) -> u32Calculate the maximum number of iterations required according to safegcd-bounds: https://github.com/sipa/safegcd-bounds