SignedInt

Struct SignedInt 

struct SignedInt<const LIMBS: usize> {
    sign: Choice,
    magnitude: Uint<LIMBS>,
}
A Uint which carries a separate sign in order to maintain the same range.

Fields

sign: Choicemagnitude: Uint<LIMBS>

Implementations

Source

impl<const LIMBS: usize> SignedInt<LIMBS>

pub const ZERO: Self

pub const fn from_uint(uint: Uint<LIMBS>) -> Self

Construct a new SignedInt from a Uint.

pub const fn from_uint_sign(magnitude: Uint<LIMBS>, sign: Choice) -> Self

Construct a new SignedInt from a Uint and a sign flag.

pub const fn magnitude(&self) -> Uint<LIMBS>

Obtain the magnitude of the SignedInt, ie. its absolute value.

pub const fn is_nonzero(&self) -> Choice

Determine if the SignedInt is non-zero.

pub const fn is_zero_vartime(&self) -> bool

Determine if the SignedInt is zero in variable time.

pub const fn is_negative(&self) -> Choice

Determine if the SignedInt is negative. Note: -0 is representable in this type, so it may be necessary to check self.is_nonzero() as well.

pub const fn lowest(&self) -> i64

Extract the lowest 63 bits and convert to its signed representation.

pub(crate) const fn lincomb_int<const RHS: usize>( a: &SignedInt<LIMBS>, b: &SignedInt<LIMBS>, c: &Int<RHS>, d: &Int<RHS>, ) -> (Uint<LIMBS>, Uint<RHS>, Choice)

Compute the linear combination a•b + c•d, returning (lo, hi, sign).

pub(crate) const fn lincomb_int_reduce_shift<const S: usize>( a: &Self, b: &Self, c: &Int<S>, d: &Int<S>, shift: u32, ) -> Self

Compute the linear combination a•b + c•d, and shift the result shift bits to the right, returning a signed value in the same range as the SignedInt inputs.

pub(crate) const fn lincomb_int_reduce_shift_mod<const S: usize>( a: &Self, b: &Self, c: &Int<S>, d: &Int<S>, shift: u32, m: &Uint<LIMBS>, mi: Uint<S>, ) -> SignedInt<LIMBS>

Compute the linear combination a•b + c•d, and shift the result shift bits to the right modulo m, returning a signed value in the same range as the SignedInt inputs.

const fn norm(&self, f_sign: Choice, m: &Uint<LIMBS>) -> Uint<LIMBS>

Normalize the value to a Uint in the range [0, m).

pub const fn eq(a: &Self, b: &Self) -> Choice

Compare two SignedInt in constant time.